%{
/*
   This file is part of the index package,
   Copyright (C) 1997 Harold Tay, haroldtay@pacific.net.sg

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA
 */
/* Scanner to get words from a tn article file.
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <ctype.h>

extern void putword(char *);
extern int getinput(char * buf, int max);
int min_word_length = 3;
int max_word_length = 15;

#define YY_INPUT(b,r,m) (r = getinput(b, m))

%}

word [a-z0-9]{3,15}
worthless1 article.*<[^@]*@[a-z0-9.]*>.*[:.]
worthless2 message.*<[^@]*@[a-z0-9.]*>.*[:.]
worthless3 post.*<[^@]*@[a-z0-9.]*>.*[:.]
contraction [a-z]+'[ts]
base64 ^[a-z0-9+/]+=?\r\n

%%

.|\n                     ;

{worthless1}       ;
{worthless2}       ;
{worthless3}       ;
{contraction}      ;
{base64} ;
the ;
and ;
for ;
you ;
that ;
wrote ;
with ;
have ;
this ;
com ;
are ;
not ;
but ;
can ;
from ;
your ;
all ;
one ;
what ;
will ;
there ;
would ;
any ;
they ;
like ;
was ;
http ;
about ;
just ;
has ;
more ;
www ;
some ;
out ;
only ;
get ;
how ;
know ;
which ;
when ;
other ;
use ;
please ;
time ;
also ;
then ;
good ;
1997 ;
who ;
than ;
does ;
them ;
now ;
very ;
want ;
need ;
should ;
been ;
see ;
could ;
way ;
anyone ;
their ;
where ;
thanks ;
into ;
because ;
here ;
these ;
had ;
much ;
why ;
most ;
may ;
used ;
many ;
same ;
two ;
too ;
first ;
problem ;
still ;
our ;
really ;
over ;
must ;
something ;
those ;
were ;
say ;
such ;
take ;
his ;
before ;
call ;
after ;
looking ;
try ;
off ;
another ;
both ;
since ;
being ;
better ;
while ;
through ;
did ;
its ;
without ;
etc ;
thing ;
got ;
writes ;
might ;
made ;
own ;
going ;
look ;
around ;
give ;
following ;
long ;
sure ;
actually ;
day ;
send ;
few ;
either ;
things ;
down ;
put ;
however ;
above ;
come ;
anything ;
said ;
tell ;
real ;
every ;
enough ;
always ;
again ;
yes ;
high ;
ever ;
says ;
great ;
let ;
least ;
probably ;
him ;
though ;
yet ;
seems ;
check ;
done ;
else ;
able ;
keep ;
possible ;
under ;
times ;
thank ;
getting ;
doing ;
maybe ;
comes ;
away ;
rather ;
having ;
big ;
written ;
once ;
less ;
hand ;
job ;
next ;
word ;
change ;
quite ;
regards ;
makes ;
open ;
seen ;
{word}                   {
                           putword(yytext);
                         }

%%

static char * body = NULL;
static char * end = NULL;
static int used = 0;
static int flag_for_yywrap = 0;

static int
copyn(char * to, char * from, int n)
{
  char * p = from;
  while( (*to++ = *from++) && n-- );
  return((from - p) - 1);
}

int
getinput(char * buf, int max)
{
  int len;
  if( body + used >= end ){
    flag_for_yywrap = 1;
    return(0);
  }
  len = copyn(buf, body + used, max);
  used += len;
  flag_for_yywrap = (len?0:1);
  return(len);
}

void
putword(char * text)
{
  char * cp;
  if( yyleng >= min_word_length && yyleng <= max_word_length ){
    for(cp = text; *cp; cp++)
      if( isupper(*cp) )*cp = tolower(*cp);
    printf("%s\n", text);
  }
}

int
yywrap(void)
{
  return(flag_for_yywrap);
}

int
main(int argc, char ** argv)
{
  int i;
  char * start = NULL;
  int argstart = 1;

  for(i = 1; i < argc; i++)
    if( '-' == *argv[i] )
      if( isdigit(argv[i][1]) ){
        char * cp;
        min_word_length = atoi(&argv[i][1]);
        cp = strchr(argv[i] + 1, '-');
        if( cp )max_word_length = atoi(cp + 1);
        argstart = 2;
      }else{
        fprintf(stderr, "Usage: %s [-n-m] files ...\nwhere "
          "n is the minimum word length, m is the max.\n", argv[0]);
        exit(1);
      }

  for(i = 1; i < argc; i++){
    struct stat st;
    int fd;
    if( -1 == stat(argv[i], &st) ){
      fprintf(stderr, "Can't stat %s\n", argv[i]);
      continue;
    }
    if( ! S_ISREG(st.st_mode) ){
      fprintf(stderr, "%s is not a regular file\n", argv[i]);
      continue;
    }
    fd = open(argv[i], O_RDONLY);
    if( -1 == fd ){
      fprintf(stderr, "Can't open %s: %s\n", argv[i], strerror(errno));
      continue;
    }
    start = mmap(0, st.st_size + 1, PROT_READ|PROT_WRITE, MAP_FILE|MAP_PRIVATE, fd, 0);
    if( (int)start <= 0 ){
      fprintf(stderr, "Can't mmap %s: %s\n", argv[i], strerror(errno));
      continue;
    }
    start[st.st_size] = '\0';  /* make sure str* functions work */
    for(body = start; body && body < start + st.st_size + 1; body = end){
      body = strstr(body, "\n\n");
      if( NULL == body )continue;
      body += 2;
      end = strstr(body, "\nFrom ");
      if( NULL == end )continue;
      end++;
      printf("\001%d\n", body - start);
      used = 0;
      yyrestart(NULL);
      yylex();
    }
    munmap(start, st.st_size + 1);
    close(fd);
  }
  exit(0);
}
